import java.io.BufferedReader;
import java.io.IOException;
import java.io.InputStreamReader;
import java.util.Arrays;

public class TypesetterArt {
    private int nRenders;
    private int nFonts;
    private int precision;
    private int width;
    private int height;
    private int[][][] layers;

    private int fontId = 0;
    private int letterId = 31;
    private final int fontSize = 8;

    public int init(int nRenders, int nFonts, int precision, int width, int height, int[] image) {
        this.nRenders = nRenders;
        this.nFonts = nFonts;
        this.precision = precision;
        this.width = width;
        this.height = height;
        this.layers = new int[4][height][width];
        
        int index = 0;
        for (int k = 0; k < 4; k++) {
            for (int i = 0; i < height; i++) {
                for (int j = 0; j < width; j++) {
                    this.layers[k][i][j] = image[index++];
                }
            }
        }
        
        return 0;
    }
    
    public int[] nextGlyph(int width, int height, int[] bitmask, int remainingMillis) {
        if (letterId < 127) {
          letterId++;
        } else {
          letterId = 32;
          fontId++;
        }
        
        if (letterId < 127 && fontId < nFonts) {
            return new int[] { letterId, fontId, fontSize };
        }
        else {
            return new int[0];
        }
    }
    
    public int[] render(int iteration, int remainingMillis) {
        if (iteration >= 4 || fontId >= nFonts) {
            return new int[] { };
        }

        return new int[] { fontId, fontSize, iteration, 255 };
    }
    
    public static void main(String[] args) throws IOException {
        TypesetterArt solution = new TypesetterArt();
        
        try (BufferedReader in = new BufferedReader(new InputStreamReader(System.in))) {
            int nRenders = Integer.parseInt(in.readLine());
            int nFonts = Integer.parseInt(in.readLine());
            int precision = Integer.parseInt(in.readLine());
            int width = Integer.parseInt(in.readLine());
            int height = Integer.parseInt(in.readLine());
            {    
                int n = Integer.parseInt(in.readLine());
                int[] image = new int[n];

                for (int i = 0; i < n; i++) {
                    image[i] = Integer.parseInt(in.readLine());
                }

                System.err.printf("Calling init\n");

                int d = solution.init(nRenders, nFonts, precision, width, height, image);

                System.err.printf("Return from init: %d\n", d);

                System.out.printf("%d\n", d);
                System.out.flush();
            }
            
            boolean callNext = true;
            while (callNext) {
                int glyphWidth = Integer.parseInt(in.readLine());
                int glyphHeight = Integer.parseInt(in.readLine());
                int n = Integer.parseInt(in.readLine());
                int[] bitmask = new int[n];
                for (int i = 0; i < n; i++) {
                    bitmask[i] = Integer.parseInt(in.readLine());
                }
                int remainingMillis = Integer.parseInt(in.readLine());
                
                System.err.printf("Calling next glyph\n");
                
                int[] ret = solution.nextGlyph(glyphWidth, glyphHeight, bitmask, remainingMillis);
                
                System.err.printf("Return from next glyph size: %d\n", ret.length);
                
                if (ret.length == 0) {
                    callNext = false;
                    System.out.printf("\n");
                }
                else {
                    System.out.printf("%d %d %d\n", ret[0], ret[1], ret[2]);
                }
                System.out.flush();
            }
            
            for (int i = 0; i < nRenders; i++) {
                int remainingMillis = Integer.parseInt(in.readLine());
                
                System.err.printf("Calling render\n");
                
                int[] ret = solution.render(i, remainingMillis);
                
                System.err.printf("Return from render size: %d\n", ret.length);
                
                System.out.printf("%d\n", ret.length);
                for (int x : ret) {
                    System.out.printf("%d\n", x);
                }
                System.out.flush();
            }
        }
    }
}